source("AMCMCUpdate.R")

#data cleaned and adapted from WA HSIS data
data <- read.csv("wacrashdat.csv")

#make x-matrix
x <- data[,c(4:14,18)]
#left shoulder type--asphalt shoulder as baseline
x$lshl_not_asphalt <- ifelse(x$lshl_typ!="A",1,0)
x$lshl_typ <- NULL
#median type--soil median as baseline
x$med_notsoil <- ifelse(x$med_type!="S",1,0)
x$med_type <- NULL
#rshl_typ--asphalt shoulder as baseline
x$rshl_not_asphalt <- ifelse(x$rshl_typ!="A",1,0)
x$rshl_typ <- NULL
#terrain--level as baseline
x$rolling <- ifelse(x$terrain=="R",1,0)
x$mountain <- ifelse(x$terrain=="M",1,0)
x$terrain <- NULL
#look at speed limit (70 as baseline)
x$spdlim60_less <- ifelse(x$spd_limt<=60,1,0)
x$spdlim65 <- ifelse(x$spd_limt==65,1,0)
x$spd_limt <- NULL
#no_lanes (5 or fewer as baseline)
#x$lanes5 <- ifelse(x$no_lanes==5,1,0)
x$lanes6 <- ifelse(x$no_lanes==6,1,0)
x$lanes7plus <- ifelse(x$no_lanes>6,1,0)
x$no_lanes <- NULL
#rural/urban rural as baseline
x$urban <- ifelse(x$rururb=="U",1,0)
x$rururb <- NULL
#medwid (leave as is)
#lshldwid (0-2 feet baseline)
x$lshl2to4 <- ifelse(x$lshldwid>2 | x$lshldwid<=4,1,0)
x$lshl4to6 <- ifelse(x$lshldwid>4 | x$lshldwid<=6,1,0)
x$lshlgreaterthan6 <- ifelse(x$lshldwid>6,1,0)
x$lshldwid <- NULL
#rshldwid (less than 10 is baseline)
x$rshl10plus <- ifelse(x$rshldwid>=10,1,0)
x$rshldwid <- NULL
#lanewid (12 or smaller as baseline)
x$lanegreaterthan12 <- ifelse(x$lanewid>12,1,0)
x$lanewid <- NULL

x <- as.matrix(x)
x <- cbind(1,x)

loglikelam <- function(lambdastar,Nk) #Nk is a vector of the number of observations (crashes) in each region
{
  mu <- exp(lambdastar)
  lambda <- Ek*mu
  -sum(lambda)+sum(Nk*log(lambda))
}

logpriorlam <- function(lambdastar,mu,Vinv)
{
  n <- length(lambdastar)
  return(-0.5*t(lambdastar-mu)%*%Vinv%*%(lambdastar-mu))
}

updatebeta <- function(y,x,m,s,sinv,Rinv)
{
  newvar <- solve(t(x)%*%Rinv%*%x + sinv)
  newmean <- newvar%*%(t(x)%*%Rinv%*%y + sinv%*%m)
  cbind(newmean,newvar)
}


n<-dim(x)[1]
p <- dim(x)[2]
nreps <- 1000000
Ek <- data$aadt/sum(data$aadt)*sum(data$counts) #expected crash counts on every segment
lambdastorage <- matrix(0,nrow=nreps/100+1,ncol=n)
Nk <- data$counts
Nk[Nk==0] <- 1
nupdatesi5 <- 0
nupdatesi90 <- 0
nupdatesi82 <- 0
nupdatesi405 <- 0
nupdatesi705 <- 0
n.badprop <- 0
betastorage <- matrix(0,nrow=nreps,ncol=p)
betastoragethin <- matrix(0,nrow=nreps/100+1,ncol=p)
gamma2storage <- numeric(nreps)

#make distance matrix
source("distancefunction.R")
D <- matrix(0,n,n)
for (i in 1:n)
{
  for(j in 1:n)
  {
    D[i,j] <- drivedistance(data$road[i],data$road[j],data$midpoint[i],data$midpoint[j])
  }
}
d <- D-t(D)
max(d)
min(d)

D2 <- matrix(0,n,n)
j <- 1
for(i in 1:n)
{
  j<-1
  while(j<=i & j<=n)
  {
    D2[i,j]<-D[i,j]
    j <- j+1
  }
}
D3 <- D2+t(D2)
D <- D3

#choose several alphas:
library(LatticeKrig)
a.seq <- seq(5,25,length=5)
omega.seq <- seq(0,.9,length=20)
aom.vals <- expand.grid(a.seq,omega.seq)
1/Matern.cor.to.range(a.seq[1],nu=1/2,cor.target=0.05)


## Calculate R matrices
Rmat <- vector("list",length=nrow(aom.vals))
for(aom in 1:nrow(aom.vals)){
  the.alph <- 1/Matern.cor.to.range(aom.vals[aom,1],nu=1/2,cor.target=0.05)
  M <- Matern(D,nu=1/2,alpha=the.alph)
  Rmat[[aom]]$R <- aom.vals[aom,2]*M+(1-aom.vals[aom,2])*diag(nrow(D))
  R.chol <- t(chol(Rmat[[aom]]$R))
  Rmat[[aom]]$Rinv <- chol2inv(t(R.chol))
  Rmat[[aom]]$R.logdet <- 2*sum(diag(R.chol))
}

updategamma2 <- function(y,x,beta,Rinv) #y is lambda star
{
  a <- n/2 + 2.1 #2.1 is prior for a
  b <- 1 + .5*t(y - x%*%beta)%*%Rinv%*%(y - x%*%beta)
  c(a,b)
}

lambdastar <- log(Nk/Ek) #lambda stars are log(mu_k)'s from the paper
c<-.003^2
prop.var <- diag(c,n)
varinv <- solve(prop.var)
lambdastorage[1,] <- exp(lambdastar)*Ek
bprmean <- matrix(0,nrow=ncol(x),ncol=1)
bprvar <- diag(10,p)
bprvarinv <- solve(bprvar)
R1.inv <- Rmat[[1]]$Rinv
gamma2storage[1] <- 1/rgamma(1,shape=2.1,rate=1)
betapost1 <- updatebeta(lambdastar,x,bprmean,bprvar,bprvarinv,(1/gamma2storage[1])*R1.inv)
betastorage[1,] <- betapost1[,1] + t(chol(betapost1[,-1]))%*%rnorm(p)
betastoragethin[1,] <- betastorage[1,]
alphastorage <- numeric(nreps-1)
omegastorage <- numeric(nreps-1)
sampstorage <- numeric(nreps-1)

## Adaptive MCMC
amcmc.it <- 250
amcmc.i5 <- list(mn=matrix(0,nrow=length(lambdastar[data$road=="i.5"]),ncol=1),
                 var=matrix(0,nrow=length(lambdastar[data$road=="i.5"]),ncol=length(lambdastar[data$road=="i.5"])))
amcmc.i90 <- list(mn=matrix(0,nrow=length(lambdastar[data$road=="i.90"]),ncol=1),
                  var=matrix(0,nrow=length(lambdastar[data$road=="i.90"]),ncol=length(lambdastar[data$road=="i.90"])))
amcmc.i82 <- list(mn=matrix(0,nrow=length(lambdastar[data$road=="i.82"]),ncol=1),
                  var=matrix(0,nrow=length(lambdastar[data$road=="i.82"]),ncol=length(lambdastar[data$road=="i.82"])))
amcmc.i405 <- list(mn=matrix(0,nrow=length(lambdastar[data$road=="i.405"]),ncol=1),
                   var=matrix(0,nrow=length(lambdastar[data$road=="i.405"]),ncol=length(lambdastar[data$road=="i.405"])))
amcmc.i705 <- list(mn=matrix(0,nrow=length(lambdastar[data$road=="i.705"]),ncol=1),
                   var=matrix(0,nrow=length(lambdastar[data$road=="i.705"]),ncol=length(lambdastar[data$road=="i.705"])))





log.dmvnorm <- function(xlist,mnvec,lstar){
  return(-0.5*xlist$R.logdet-0.5*t(lstar-mnvec)%*%xlist$Rinv%*%(lstar-mnvec))
}

## Convert log(prob) to prob
lprob.to.prob <- function(log.ai){
  log.a0 <- max(log.ai)
  log.sum.probs <- log.a0+log(sum(exp(log.ai-log.a0)))
  log.probs <- log.ai-log.sum.probs
  return(exp(log.probs))
}



k <- 2
system.time(for(i in 2:nreps)
{
  print(i)
  ##sample alpha and get R
  mu <- x%*%betastorage[i-1,]
  logprobs <- sapply(Rmat,log.dmvnorm,mnvec=mu,lambdastar)
  probs <- lprob.to.prob(logprobs)
  samp <- sample(nrow(aom.vals),1,prob=probs)
  sampstorage[i-1] <- samp
  alphastorage[i-1] <- aom.vals[samp,1]
  omegastorage[i-1] <- aom.vals[samp,2]
  R <- Rmat[[samp]]$R
  Rinv <- Rmat[[samp]]$Rinv
  
  ## Update Lambdastar sequentially, for each interstate
  #i.5
  b <- sum(data$road=="i.5")
  if(i<=amcmc.it){
    proposali5 <- lambdastar[data$road=="i.5"] + t(chol(prop.var))[1:b,1:b]%*%rnorm(b)
  } else {
    proposali5 <- lambdastar[data$road=="i.5"] + t(chol((2.4^2/nrow(amcmc.i5$var))*(amcmc.i5$var+prop.var[1:b,1:b])))%*%rnorm(b)
  }
  proposal <- lambdastar
  proposal[data$road=="i.5"] <- proposali5
  log.mh.ratio <- loglikelam(proposal,Nk) + logpriorlam(proposal,x%*%betastorage[i-1,],(1/gamma2storage[i-1])*Rinv) - loglikelam(lambdastar,Nk) - 
    logpriorlam(lambdastar,x%*%betastorage[i-1,],(1/gamma2storage[i-1])*Rinv)
  if(log(runif(1))< log.mh.ratio)
  {
    lambdastar <- proposal
    nupdatesi5 <- nupdatesi5 + 1
  }
  
  #i.90
  b <- sum(data$road=="i.90")
  if(i<=amcmc.it){
    proposali90 <- lambdastar[data$road=="i.90"] + t(chol(prop.var))[1:b,1:b]%*%rnorm(b)
  } else {
    proposali90 <- lambdastar[data$road=="i.90"] + t(chol((2.4^2/nrow(amcmc.i90$var))*(amcmc.i90$var+prop.var[1:b,1:b])))%*%rnorm(b)
  }
  proposal <- lambdastar
  proposal[data$road=="i.90"] <- proposali90
  log.mh.ratio <- loglikelam(proposal,Nk) + logpriorlam(proposal,x%*%betastorage[i-1,],(1/gamma2storage[i-1])*Rinv) - loglikelam(lambdastar,Nk) - 
    logpriorlam(lambdastar,x%*%betastorage[i-1,],(1/gamma2storage[i-1])*Rinv)
  if(log(runif(1))< log.mh.ratio)
  {
    lambdastar <- proposal
    nupdatesi90 <- nupdatesi90 + 1
  }
  
  #i.82
  b <- sum(data$road=="i.82")
  if(i<=amcmc.it){
    proposali82 <- lambdastar[data$road=="i.82"] + t(chol(prop.var))[1:b,1:b]%*%rnorm(b)
  } else {
    proposali82 <- lambdastar[data$road=="i.82"] + t(chol((2.4^2/nrow(amcmc.i82$var))*(amcmc.i82$var+prop.var[1:b,1:b])))%*%rnorm(b)
  }
  proposal <- lambdastar
  proposal[data$road=="i.82"] <- proposali82
  log.mh.ratio <- loglikelam(proposal,Nk) + logpriorlam(proposal,x%*%betastorage[i-1,],(1/gamma2storage[i-1])*Rinv) - loglikelam(lambdastar,Nk) - 
    logpriorlam(lambdastar,x%*%betastorage[i-1,],(1/gamma2storage[i-1])*Rinv)
  if(log(runif(1))< log.mh.ratio)
  {
    lambdastar <- proposal
    nupdatesi82 <- nupdatesi82 + 1
  }
  
  #i.405
  b <- sum(data$road=="i.405")
  if(i<=amcmc.it){
    proposali405 <- lambdastar[data$road=="i.405"] + t(chol(prop.var))[1:b,1:b]%*%rnorm(b)
  } else {
    proposali405 <- lambdastar[data$road=="i.405"] + t(chol((2.4^2/nrow(amcmc.i405$var))*(amcmc.i405$var+prop.var[1:b,1:b])))%*%rnorm(b)
  }
  proposal <- lambdastar
  proposal[data$road=="i.405"] <- proposali405
  log.mh.ratio <- loglikelam(proposal,Nk) + logpriorlam(proposal,x%*%betastorage[i-1,],(1/gamma2storage[i-1])*Rinv) - loglikelam(lambdastar,Nk) - 
    logpriorlam(lambdastar,x%*%betastorage[i-1,],(1/gamma2storage[i-1])*Rinv)
  if(log(runif(1))< log.mh.ratio)
  {
    lambdastar <- proposal
    nupdatesi405 <- nupdatesi405 + 1
  }
  
  #i.705
  b <- sum(data$road=="i.705")
  if(i<=amcmc.it){
    proposali705 <- lambdastar[data$road=="i.705"] + t(chol(prop.var))[1:b,1:b]%*%rnorm(b)
  } else {
    proposali705 <- lambdastar[data$road=="i.705"] + t(chol((2.4^2/nrow(amcmc.i705$var))*(amcmc.i705$var+prop.var[1:b,1:b])))%*%rnorm(b)
  }
  proposal <- lambdastar
  proposal[data$road=="i.705"] <- proposali705
  log.mh.ratio <- loglikelam(proposal,Nk) + logpriorlam(proposal,x%*%betastorage[i-1,],(1/gamma2storage[i-1])*Rinv) - loglikelam(lambdastar,Nk) - 
    logpriorlam(lambdastar,x%*%betastorage[i-1,],(1/gamma2storage[i-1])*Rinv)
  if(log(runif(1))< log.mh.ratio)
  {
    lambdastar <- proposal
    nupdatesi705 <- nupdatesi705 + 1
  }
  
  
  ## Update beta vector
  betapost <- updatebeta(lambdastar,x,bprmean,bprvar,bprvarinv,(1/gamma2storage[i-1])*Rinv)
  betastorage[i,] <- betapost[,1] + t(chol(betapost[,-1]))%*%rnorm(p)
  
  if(i/100 == as.integer(i/100))
  {
    betastoragethin[k,] <- betastorage[i,]
    mu <- exp(lambdastar)
    lambdastorage[k,] <- Ek*mu 
    k <- k+1
  }
  #lambdastorage[i,] <- pmax(0,lambdastar)/sum(pmax(0,lambdastar))
  
  ## Update Adaptive Stuff
  amcmc.i5 <- AMCMC.update(lambdastar[data$road=="i.5"],amcmc.i5$mn,amcmc.i5$var,i-1)
  amcmc.i90 <- AMCMC.update(lambdastar[data$road=="i.90"],amcmc.i90$mn,amcmc.i90$var,i-1)
  amcmc.i82 <- AMCMC.update(lambdastar[data$road=="i.82"],amcmc.i82$mn,amcmc.i82$var,i-1)
  amcmc.i405 <- AMCMC.update(lambdastar[data$road=="i.405"],amcmc.i405$mn,amcmc.i405$var,i-1)
  amcmc.i705 <- AMCMC.update(lambdastar[data$road=="i.705"],amcmc.i705$mn,amcmc.i705$var,i-1)
  
  #update gamma^2
  gammapost <- updategamma2(lambdastar,x,betastorage[i,],Rinv)
  gamma2storage[i] <- 1/rgamma(1,shape=gammapost[1],rate=gammapost[2])
}
)
lambdastorage2cR <- lambdastorage
alphastorage2cR <- alphastorage
gamma2storage2cR <- gamma2storage
betastoragethin2cR <- betastoragethin
n.badprop2cR <- n.badprop
nupdatesi5.2cR <- nupdatesi5
nupdatesi90.2cR <- nupdatesi90
nupdatesi82.2cR <- nupdatesi82
nupdatesi405.2cR <- nupdatesi405
nupdatesi705.2cR <- nupdatesi705
omegastoragecR <- omegastorage
sampstoragecR <- sampstorage
save(file="./identifyingriskfactors.RData",list=c("lambdastorage2cR","gamma2storage2cR","alphastorage2cR","omegastoragecR","sampstoragecR","betastoragethin2cR","n.badprop2cR","nupdatesi5.2cR","nupdatesi90.2cR","nupdatesi82.2cR","nupdatesi405.2cR","nupdatesi705.2cR"))

if(FALSE)
{
  load("identifyingriskfactors.RData")
  
  #check acceptance rates
  nupdatesi5.2cR/nreps
  nupdatesi90.2cR/nreps
  nupdatesi82.2cR/nreps
  nupdatesi405.2cR/nreps
  nupdatesi705.2cR/nreps
  
  table(alphastorage2cR)
  table(omegastoragecR)
  table(sampstoragecR)
  
  
  plot(1:length(alphastorage2cR),alphastorage2cR,type="l")
  plot(1:length(omegastoragecR),omegastoragecR,type="l")
  plot(1:length(gamma2storage2cR[seq(1,1000000,by=100)]),gamma2storage2cR[seq(1,1000000,by=100)],type="l")
  plot(betastoragethin2cR[,15],type="l")
  plot(lambdastorage2cR[,250],type="l")
  
  #burn
  lambdastorage2cR <- lambdastorage2cR[5001:10001,]
  betastoragethin2cR <- betastoragethin2cR[1001:10001,]
  
  par(mar=c(5,5,5,5))
  plot(betastoragethin2cR[,6],type="l",ylab=expression(paste(beta, " for Average Curvature")))
  plot(lambdastorage2cR[,50],type="l",ylab=expression(paste("log(", mu[50], ")")))
  
  plot(density(betastoragethin2cR[,1]),main="Intercept")
  plot(density(betastoragethin2cR[,2]),main="Median Width")
  plot(density(betastoragethin2cR[,3]),main="Avg Curvature")
  plot(density(betastoragethin2cR[,4]),main="Left Shoulder Not Asphalt")
  plot(density(betastoragethin2cR[,5]),main="Median Not Soil")
  plot(density(betastoragethin2cR[,6]),main="Right Shoulder Not Asphalt")
  plot(density(betastoragethin2cR[,7]),main="Rolling")
  plot(density(betastoragethin2cR[,8]),main="Mountainous")
  plot(density(betastoragethin2cR[,9]),main="Speed Limit <=60")
  plot(density(betastoragethin2cR[,10]),main="Speed Limit 65")
  plot(density(betastoragethin2cR[,11]),main="6 Lanes")
  plot(density(betastoragethin2cR[,12]),main="7 or More Lanes")
  plot(density(betastoragethin2cR[,13]),main="Urban")
  plot(density(betastoragethin2cR[,14]),main="2 to 4 ft left shoulder")
  plot(density(betastoragethin2cR[,15]),main="4 to 6 ft left shoulder")
  plot(density(betastoragethin2cR[,16]),main=">6 ft left shoulder")
  plot(density(betastoragethin2cR[,17]),main=">=10 ft right shoulder")
  plot(density(betastoragethin2cR[,18]),main=">12 ft lane width")
  
  
  #intercept
  q1 <- quantile(betastoragethin2cR[,1],c(.025,.975))
  p1 <- mean(betastoragethin2cR[,1]<0)
  #median width
  q2 <- quantile(betastoragethin2cR[,2],c(.025,.975))
  p2 <- mean(betastoragethin2cR[,2]<0)
  #avg curv
  q3 <- quantile(betastoragethin2cR[,3],c(.025,.975))
  p3 <- mean(betastoragethin2cR[,3]>0)
  #l shoulder not asphalt
  q4 <- quantile(betastoragethin2cR[,4],c(.025,.975))
  p4 <- mean(betastoragethin2cR[,4]>0)
  #median not soil
  q5 <- quantile(betastoragethin2cR[,5],c(.025,.975))
  p5 <- mean(betastoragethin2cR[,5]<0)
  #r shoulder not asphalt
  q6 <- quantile(betastoragethin2cR[,6],c(.025,.975))
  p6 <- mean(betastoragethin2cR[,6]>0)
  #rolling
  q7 <- quantile(betastoragethin2cR[,7],c(.025,.975))
  p7 <- mean(betastoragethin2cR[,7]>0)
  #mountain
  q8 <- quantile(betastoragethin2cR[,8],c(.025,.975))
  p8 <- mean(betastoragethin2cR[,8]>0)
  #spd lim 60 or less
  q9 <- quantile(betastoragethin2cR[,9],c(.025,.975))
  p9 <- mean(betastoragethin2cR[,9]>0)
  #spd lim 65
  q10 <- quantile(betastoragethin2cR[,10],c(.025,.975))
  p10 <- mean(betastoragethin2cR[,10]>0)
  #6 lanes
  q11 <- quantile(betastoragethin2cR[,11],c(.025,.975))
  p11 <- mean(betastoragethin2cR[,11]<0)
  #7 + lanes
  q12 <- quantile(betastoragethin2cR[,12],c(.025,.975))
  p12 <- mean(betastoragethin2cR[,12]<0)
  #urban
  q13 <- quantile(betastoragethin2cR[,13],c(.025,.975))
  p13 <- mean(betastoragethin2cR[,13]<0)
  #2-4 ft left shoulder
  q14 <- quantile(betastoragethin2cR[,14],c(.025,.975))
  p14 <- mean(betastoragethin2cR[,14]<0)
  #4-6 ft left shoulder
  q15 <- quantile(betastoragethin2cR[,15],c(.025,.975))
  p15 <- mean(betastoragethin2cR[,15]<0)
  # 6 + ft left shoulder
  q16 <- quantile(betastoragethin2cR[,16],c(.025,.975))
  p16 <- mean(betastoragethin2cR[,16]>0)
  #10 + ft right shoulder
  q17 <- quantile(betastoragethin2cR[,17],c(.025,.975))
  p17 <- mean(betastoragethin2cR[,17]<0)
  #lane width >12 ft
  q18 <- quantile(betastoragethin2cR[,18],c(.025,.975))
  p18 <- mean(betastoragethin2cR[,18]>0)
  
  
  pvals <- c(p1,p2,p3,p4,p5,p6,p7,p8,p9,p10,p11,p12,p13,p14,p15,p16,p17,p18)
  quants <- rbind(q1,q2,q3,q4,q5,q6,q7,q8,q9,q10,q11,q12,q13,q14,q15,q16,p17,p18)
  betapostmean <- apply(betastoragethin2cR,2,mean)
  nams <- c("Intercept","Median Width","Average Curvature","Left Shoulder Not Asphalt", "Median Not Soil",
            "Right Shoulder Not Asphalt", "Rolling Terrain", "Mountainous Terrain", 
            "Speed Limit of 60 mph or less", "Speed Limit 65 mph", "6 Lanes", "7+ Lanes",
            "Urban","2-4 ft Left Shoulder","4-6 ft Left Shoulder",">6 ft Left Shoulder",
            "10+ ft Right Shoulder", "Lane Width > 12 ft")
  results <- data.frame(nams,betapostmean,quants,pvals)
  results2 <- results[order(pvals,decreasing=T),]
  library(xtable)
  xtable(results2)
  
  #look at residuals
  betapostmean <- apply(betastoragethin2cR,2,mean)
  xbetapost <- x%*%betapostmean
  
  lambdastarcheck <- matrix(0,nrow=nrow(lambdastorage2cR),ncol=ncol(lambdastorage2cR))
  for(i in 1:nrow(lambdastarcheck))
  {
    lambdastarcheck[i,] <- log(lambdastorage2cR[i,]/Ek)
  }
  
  lambdastarpostmean <- apply(lambdastarcheck,2,mean)
  
  
  resids <- lambdastarpostmean - xbetapost
  
  plot(1:741,resids)
  scatter.smooth(1:741,resids,degree=2)
  dim(i5data)
  
  par(mfrow=c(1,2),mar=c(4.5,4.5,3,2))
  #i-5
  scatter.smooth(1:277,resids[1:277],degree=2,main="Spatial Residuals on I-5",xlab="Mile",ylab="Residuals",ylim=c(-1,2.5))
  abline(v=c(105,133,164),col=c("green","cornflowerblue","red"),lwd=5,lty=c(1,2,3))
  legend("topleft",legend=c("Olympia","Tacoma","Seattle"),col=c("green","cornflowerblue","red"),lwd=5,lty=c(1,2,3))

  
  #i-90
  index <- data$road=="i.90"
  scatter.smooth(1:sum(index),resids[index],degree=2,main="Spatial Residuals on I-90",xlab="Mile",ylab="Residuals",ylim=c(-1.5,2))
  abline(v=c(2,280),col=c("red","orange"),lwd=5,lty=c(3,2))
  legend("top",legend=c("Seattle","Spokane"),col=c("red","orange"),lwd=5,lty=c(3,2))

  #i-82
  index <- data$road=="i.82"
  plot(1:sum(index),resids[index],main="Residuals on I-82",type='l')
  scatter.smooth(1:sum(index),resids[index],degree=2,main="Residuals on I-82")
  #i-405
  index <- data$road=="i.405"
  plot(1:sum(index),resids[index],main="Residuals on I-405",type='l')
  scatter.smooth(1:sum(index),resids[index],degree=2,main="Residuals on I-405")
  #i-705
  index <- data$road=="i.705"
  plot(1:sum(index),resids[index],main="Residuals on I-705")
  #scatter.smooth(1:sum(index),resids[index],degree=2,main="Residuals on I-705")
  
  #plot mus
  mucheck <- matrix(0,nrow=nrow(lambdastorage2cR),ncol=ncol(lambdastorage2cR))
  for(i in 1:nrow(mucheck))
  {
    mucheck[i,] <- lambdastorage2cR[i,]/Ek
  }
  mupostmean <- apply(mucheck,2,mean)
  
  plot(1:277,mupostmean[1:277],main="Relative Risk on I-5",type='l',xlab='Mile',ylab='Mu Posterior Mean')
  abline(v=c(105,133,164),lty=c(1,2,3),col=c("green","cornflowerblue","red"),lwd=5)
  lines(1:277,mupostmean[1:277])
  legend("topleft",legend=c("Olympia","Tacoma","Seattle"),col=c("green","cornflowerblue","red"),lwd=5,lty=c(1,2,3))
  
  index <- data$road=="i.90"
  plot(1:sum(index),mupostmean[index],main="Relative Risk on I-90",type='l',xlab='Mile',ylab='Mu Posterior Mean')
  abline(v=c(2,280),col=c("red","orange"),lwd=5,lty=c(3,2))
  lines(1:sum(index),mupostmean[index])
  legend("top",legend=c("Seattle","Spokane"),col=c("red","orange"),lwd=5,lty=c(3,2))
  
  index <- data$road=="i.82"
  plot(1:sum(index),mupostmean[index],main="Relative Risk on I-82",type='l')
  index <- data$road=="i.405"
  plot(1:sum(index),mupostmean[index],main="Relative Risk on I-405",type='l')
  index <- data$road=="i.705"
  plot(1:sum(index),mupostmean[index],main="Relative Risk on I-705",type='l')
  
  
  postmean2 <- apply(lambdastorage2cR,2,mean)
  plot(postmean2,Nk,xlab="Expected Crash Counts",ylab="Observed Crash Counts",main="Expected vs. Observed Counts")
  cor(postmean2,Nk)
}
